@@ -4,7 +4,7 @@
 #
 # Handle coxph regression model.
 #
-# Time-stamp: <2009-11-04 14:01:29 Graham Williams>
+# Time-stamp: <2009-11-23 20:15:16 Graham Williams>
 #
 # Copyright (c) 2009 Togaware Pty Ltd
 #
@@ -150,6 +150,7 @@ pmml.coxph <- function(model,
 
   coeff <- coefficients(model)
   coeffnames <- names(coeff)
+  means <- model$means
 
   intercept <- 0
   
@@ -161,6 +162,10 @@ pmml.coxph <- function(model,
   # CategoricalPredictors. Simplest approach is to loop twice!!
   # Hopefully, this is not a significant computational expense.
 
+  # For the coxph regression, we need to record the means. This is
+  # then used to subtract from supplied value before multiplying by
+  # the coefficient in the regression formula.
+
   for (i in 1:length(orig.names))
   {
     name <- orig.names[[i]]
@@ -171,7 +176,8 @@ pmml.coxph <- function(model,
       predictorNode <- xmlNode("NumericPredictor",
                                attrs=c(name=name,
                                  exponent="1",
-                                 coefficient=as.numeric(coeff[which(coeffnames==name)])))
+                                 coefficient=as.numeric(coeff[which(coeffnames==name)]),
+                                 mean=as.numeric(means[which(coeffnames==name)])))
       regTable <- append.XMLNode(regTable, predictorNode)
     }
   }
