@@ -4,7 +4,7 @@
 #
 # Handle lm and glm models.
 #
-# Time-stamp: <2010-10-09 06:12:11 Graham Williams>
+# Time-stamp: <2011-01-01 11:42:53 Graham Williams>
 #
 # Copyright (c) 2009 Togaware Pty Ltd
 #
@@ -40,6 +40,8 @@ pmml.lm <- function(model,
                     description="Linear Regression Model",
                     copyright=NULL,
                     transforms=NULL,
+                    dataset=NULL,
+                    weights=NULL,
                     ...)
 {
   if (! inherits(model, "lm")) stop("Not a legitimate lm object")
@@ -128,7 +130,7 @@ pmml.lm <- function(model,
 
   # PMML -> DataDictionary
 
-  pmml <- append.XMLNode(pmml, pmmlDataDictionary(field))
+  pmml <- append.XMLNode(pmml, pmmlDataDictionary(field, weights=weights))
 
   # PMML -> RegressionModel
 
